@@ -1,3 +1,4 @@
+use std::fmt::Display;
 use std::{io::Error, str::FromStr};
 
 use proc_macro2::{Ident, Span, TokenStream as TokenStream2};
@@ -338,7 +339,7 @@ impl ToTokens for Path {
             .path_attr
             .path_operation
             .as_ref()
-            .or_else(|| self.path_operation.as_ref())
+            .or(self.path_operation.as_ref())
             .unwrap_or_else(|| {
                 #[cfg(feature = "actix_extras")]
                 let help =
@@ -358,7 +359,7 @@ impl ToTokens for Path {
             .path_attr
             .path
             .as_ref()
-            .or_else(|| self.path.as_ref())
+            .or(self.path.as_ref())
             .unwrap_or_else(|| {
                 #[cfg(feature = "actix_extras")]
                 let help =
@@ -492,10 +493,10 @@ impl ToTokens for Operation<'_> {
 }
 
 trait ContentTypeResolver {
-    fn resolve_content_type<'a>(
+    fn resolve_content_type<'a, T: Display>(
         &self,
         content_type: Option<&'a String>,
-        component_type: &ComponentType<'a>,
+        component_type: &ComponentType<'a, T>,
     ) -> &'a str {
         if let Some(content_type) = content_type {
             content_type
